@@ -13,6 +13,7 @@ from .eou import EOU
 from .job import get_current_job_context
 from .denoise import Denoise
 import logging
+import asyncio
 
 logger = logging.getLogger(__name__)
 
@@ -203,15 +204,51 @@ class CascadingPipeline(Pipeline, EventEmitter[Literal["error"]]):
         Handle user speech started event
         """
         self._notify_speech_started()
+    
+    def interrupt(self) -> None:
+        """
+        Interrupt the pipeline
+        """
+        if self.conversation_flow:
+            asyncio.create_task(self.conversation_flow._interrupt_tts())
+    
 
     async def cleanup(self) -> None:
         """Cleanup all pipeline components"""
+        logger.info("Cleaning up cascading pipeline")
         if self.stt:
             await self.stt.aclose()
+            self.stt = None
         if self.llm:
             await self.llm.aclose()
+            self.llm = None
         if self.tts:
             await self.tts.aclose()
+            self.tts = None
+        if self.vad:
+            await self.vad.aclose()
+            self.vad = None
+        if self.turn_detector:
+            await self.turn_detector.aclose()
+            self.turn_detector = None
+        if self.denoise:
+            await self.denoise.aclose()
+            self.denoise = None        
+        if self.conversation_flow:
+            try:
+                await self.conversation_flow.cleanup()
+            except Exception as e:
+                logger.error(f"Error cleaning up conversation flow: {e}")
+        
+        self.agent = None
+        self.conversation_flow = None
+        self.avatar = None
+        logger.info("Cascading pipeline cleaned up")
+        await super().cleanup()
+    
+    async def leave(self) -> None:
+        """Leave the cascading pipeline"""
+        await self.cleanup()
 
     def get_component_configs(self) -> dict[str, dict[str, Any]]:
         """Return a dictionary of component configurations (STT, LLM, TTS) with their instance attributes.
@@ -318,3 +355,16 @@ class CascadingPipeline(Pipeline, EventEmitter[Literal["error"]]):
 
         cascading_metrics_collector.add_error(source, str(error_data))
         logger.error(f"[{source}] Component error: {error_data}")
+
+    async def reply_with_context(self, instructions: str, wait_for_playback: bool = True) -> None:
+        """
+        Generate a reply using instructions and current chat context.
+        
+        Args:
+            instructions: Instructions to add to chat context
+            wait_for_playback: If True, disable VAD and STT interruptions during response and wait for
+        """
+        if self.conversation_flow:
+            await self.conversation_flow._process_reply_instructions(instructions, wait_for_playback)
+        else:
+            logger.warning("No conversation flow found in pipeline")
