@@ -18,6 +18,7 @@ from .vad import VAD, VADResponse, VADEventType
 from .eou import EOU
 from .metrics import cascading_metrics_collector
 from .denoise import Denoise
+from .utils import UserState, AgentState
 import logging
 
 logger = logging.getLogger(__name__)
@@ -57,6 +58,15 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
         self._partial_response = ""
         self._is_interrupted = False
 
+        # Enhanced transcript accumulation system
+        self._accumulated_transcript = ""
+        self._waiting_for_more_speech = False
+        self._speech_wait_timeout = 0.8  # 800ms timeout
+        self._wait_timer: asyncio.TimerHandle | None = None
+        self._transcript_processing_lock = asyncio.Lock()
+
+        # self._eou_timer_task: asyncio.Task | None = None
+
     async def start(self) -> None:
         global_event_emitter.on("speech_started", self.on_speech_started_stt)
         global_event_emitter.on("speech_stopped", self.on_speech_stopped_stt)
@@ -96,15 +106,103 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
     async def on_vad_event(self, vad_response: VADResponse) -> None:
         """Handle VAD events"""
         if vad_response.event_type == VADEventType.START_OF_SPEECH:
+            # If we're waiting for more speech and user starts speaking again
+            if self._waiting_for_more_speech:
+                await self._handle_continued_speech()
             await self.on_speech_started()
         elif vad_response.event_type == VADEventType.END_OF_SPEECH:
             self.on_speech_stopped()
 
+    async def _handle_continued_speech(self) -> None:
+        """Handle when user continues speaking while we're waiting"""
+        # Cancel the wait timer
+        if self._wait_timer:
+            self._wait_timer.cancel()
+            self._wait_timer = None
+        
+        self._waiting_for_more_speech = False
+
     async def on_stt_transcript(self, stt_response: STTResponse) -> None:
-        """Handle STT transcript events"""
+        """Handle STT transcript events with enhanced EOU logic"""
         if stt_response.event_type == SpeechEventType.FINAL:
             user_text = stt_response.data.text
-            await self._process_final_transcript(user_text)
+            await self._process_transcript_with_eou(user_text)
+
+    async def _process_transcript_with_eou(self, new_transcript: str) -> None:
+        """Enhanced transcript processing with EOU-based decision making"""
+        async with self._transcript_processing_lock:
+            # Append new transcript to accumulated transcript
+            if self._accumulated_transcript:
+                self._accumulated_transcript += " " + new_transcript
+            else:
+                self._accumulated_transcript = new_transcript
+            
+            # Check EOU with accumulated transcript
+            is_eou = await self._check_end_of_utterance(self._accumulated_transcript)
+            
+            if is_eou:
+                await self._finalize_transcript_and_respond()
+            else:
+                await self._wait_for_additional_speech()
+
+    async def _check_end_of_utterance(self, transcript: str) -> bool:
+        """Check if the current transcript represents end of utterance"""
+        if not self.turn_detector:
+            # If no EOU detector, assume it's always end of utterance
+            return True
+        
+        # Create temporary chat context for EOU detection
+        temp_context = self.agent.chat_context.copy()
+        temp_context.add_message(role=ChatRole.USER, content=transcript)
+        
+        cascading_metrics_collector.on_eou_start()
+        is_eou = self.turn_detector.detect_end_of_utterance(temp_context)
+        cascading_metrics_collector.on_eou_complete()
+        
+        return is_eou
+
+    async def _wait_for_additional_speech(self) -> None:
+        """Wait for additional speech within the timeout period"""
+
+        if self._waiting_for_more_speech:
+            # Already waiting, extend the timer
+            if self._wait_timer:
+                self._wait_timer.cancel()
+        
+        self._waiting_for_more_speech = True
+        
+        # Set timer for speech timeout
+        loop = asyncio.get_event_loop()
+        self._wait_timer = loop.call_later(
+            self._speech_wait_timeout,
+            lambda: asyncio.create_task(self._on_speech_timeout())
+        )
+        
+
+    async def _on_speech_timeout(self) -> None:
+        """Handle timeout when no additional speech is detected"""
+        async with self._transcript_processing_lock:
+            if not self._waiting_for_more_speech:
+                return  # Already processed or cancelled
+            
+            self._waiting_for_more_speech = False
+            self._wait_timer = None
+            
+            await self._finalize_transcript_and_respond()
+
+    async def _finalize_transcript_and_respond(self) -> None:
+        """Finalize the accumulated transcript and generate response"""
+        if not self._accumulated_transcript.strip():
+            return
+        
+        final_transcript = self._accumulated_transcript.strip()
+        logger.info(f"Finalizing transcript: '{final_transcript}'")
+        
+        # Reset accumulated transcript
+        self._accumulated_transcript = ""
+        
+        # Process the final transcript
+        await self._process_final_transcript(final_transcript)
 
     async def _process_final_transcript(self, user_text: str) -> None:
         """Process final transcript with EOU detection and response generation"""
@@ -129,23 +227,78 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
 
         await self.on_turn_start(user_text)
 
-        if self.turn_detector:
-            cascading_metrics_collector.on_eou_start()
-            eou_detected = self.turn_detector.detect_end_of_utterance(
-                self.agent.chat_context)
-            cascading_metrics_collector.on_eou_complete()
+        # Generate response
+        asyncio.create_task(self._generate_and_synthesize_response(user_text))
 
-            if eou_detected:
-                asyncio.create_task(
-                    self._generate_and_synthesize_response(user_text))
-            else:
-                cascading_metrics_collector.complete_current_turn()
-        else:
-            asyncio.create_task(
-                self._generate_and_synthesize_response(user_text))
+        # Async helper: waits before generating a response (used if utterance isn't clearly ended)
+        # async def generate_response_after_delay(delay: float):
+        #     await asyncio.sleep(delay)
+        #     if not asyncio.current_task().done():
+        #         await self._generate_and_synthesize_response(user_text)
+
+        # If turn detection is enabled
+        # if self.turn_detector:
+        #     cascading_metrics_collector.on_eou_start()
+        #     eou_detected = self.turn_detector.detect_end_of_utterance(
+        #         self.agent.chat_context)
+        #     cascading_metrics_collector.on_eou_complete()
+
+        #     If user finished speaking → respond immediately
+        #     if eou_detected:
+        #         asyncio.create_task(
+        #             self._generate_and_synthesize_response(user_text))
+        #     Else → start a 2s timer, then respond if no speech continues
+        #     else:
+        #         self._eou_timer_task = asyncio.create_task(generate_response_after_delay(2.0))
+        #         # cascading_metrics_collector.complete_current_turn()
+        # else:
+        #     # If no turn detection, always respond immediately
+        #     asyncio.create_task(
+        #         self._generate_and_synthesize_response(user_text))
 
         await self.on_turn_end()
 
+    async def _process_reply_instructions(self, instructions: str, wait_for_playback: bool = True) -> None:
+        """Process reply instructions and generate response using existing flow"""
+        
+        original_vad_handler = None
+        original_stt_handler = None
+        
+        if wait_for_playback:
+            # Temporarily disable VAD events
+            if self.vad:
+                original_vad_handler = self.on_vad_event
+                self.on_vad_event = lambda x: None
+            
+            # Temporarily disable STT transcript processing
+            if self.stt:
+                original_stt_handler = self.on_stt_transcript
+                self.on_stt_transcript = lambda x: None
+        
+        try:
+            self.agent.chat_context.add_message(
+                role=ChatRole.USER,
+                content=instructions
+            )
+
+            await self.on_turn_start(instructions)
+            await self._generate_and_synthesize_response(instructions)
+
+            await self.on_turn_end()
+            
+            if wait_for_playback:
+                while (hasattr(cascading_metrics_collector.data, 'is_agent_speaking') and 
+                    cascading_metrics_collector.data.is_agent_speaking):
+                    await asyncio.sleep(0.1)
+                    
+        finally:
+            if wait_for_playback:
+                if original_vad_handler is not None:
+                    self.on_vad_event = original_vad_handler
+                
+                if original_stt_handler is not None:
+                    self.on_stt_transcript = original_stt_handler
+
     async def _generate_and_synthesize_response(self, user_text: str) -> None:
         """Generate agent response"""
         self._is_interrupted = False
@@ -343,11 +496,14 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
     def on_speech_started_stt(self, event_data: Any) -> None:
         if self.user_speech_callback:
             self.user_speech_callback()
+        
+        if self.agent.session:
+            self.agent.session._emit_user_state(UserState.SPEAKING)
 
     def on_speech_stopped_stt(self, event_data: Any) -> None:
         pass
 
-    async def on_speech_started(self) -> None:
+    async def on_speech_started(self) -> None:       
         cascading_metrics_collector.on_user_speech_start()
 
         if self.user_speech_callback:
@@ -358,12 +514,22 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
 
         if self.tts:
             await self._interrupt_tts()
+        
+        if self.agent.session:
+            self.agent.session._emit_user_state(UserState.SPEAKING)
+            self.agent.session._emit_agent_state(AgentState.LISTENING)
 
     async def _interrupt_tts(self) -> None:
         logger.info("Interrupting TTS and LLM generation")
 
         self._is_interrupted = True
 
+        # Cancel any waiting timers
+        if self._wait_timer:
+            self._wait_timer.cancel()
+            self._wait_timer = None
+        self._waiting_for_more_speech = False
+
         if self.tts:
             await self.tts.interrupt()
 
@@ -394,6 +560,10 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
             self._stt_started = True
 
         cascading_metrics_collector.on_user_speech_end()
+        
+        if self.agent.session:
+            self.agent.session._emit_user_state(UserState.IDLE)
+            self.agent.session._emit_agent_state(AgentState.THINKING)
 
     async def _synthesize_with_tts(self, response_gen: AsyncIterator[str] | str) -> None:
         """
@@ -401,10 +571,16 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
         """
         if not self.tts:
             return
+        
+        self.agent.session._pause_wake_up_timer()
 
         async def on_first_audio_byte():
             cascading_metrics_collector.on_tts_first_byte()
             cascading_metrics_collector.on_agent_speech_start()
+            
+            if self.agent.session:
+                self.agent.session._emit_agent_state(AgentState.SPEAKING)
+                self.agent.session._emit_user_state(UserState.LISTENING)
 
         self.tts.on_first_audio_byte(on_first_audio_byte)
         self.tts.reset_first_audio_tracking()
@@ -420,6 +596,61 @@ class ConversationFlow(EventEmitter[Literal["transcription"]], ABC):
                 response_iterator = response_gen
 
             await self.tts.synthesize(response_iterator)
+            
 
         finally:
+            self.agent.session._reply_in_progress = False
+            self.agent.session._reset_wake_up_timer()
             cascading_metrics_collector.on_agent_speech_end()
+            
+            if self.agent.session:
+                self.agent.session._emit_agent_state(AgentState.IDLE)
+                self.agent.session._emit_user_state(UserState.IDLE)
+    
+    async def cleanup(self) -> None:
+        """Cleanup conversation flow resources"""
+        logger.info("Cleaning up conversation flow")
+        if self._current_tts_task and not self._current_tts_task.done():
+            self._current_tts_task.cancel()
+            try:
+                await self._current_tts_task
+            except asyncio.CancelledError:
+                pass
+            self._current_tts_task = None
+        
+        if self._current_llm_task and not self._current_llm_task.done():
+            self._current_llm_task.cancel()
+            try:
+                await self._current_llm_task
+            except asyncio.CancelledError:
+                pass
+            self._current_llm_task = None
+        
+        if self._eou_timer_task and not self._eou_timer_task.done():
+            self._eou_timer_task.cancel()
+            try:
+                await self._eou_timer_task
+            except asyncio.CancelledError:
+                pass
+            self._eou_timer_task = None
+        
+        if hasattr(self, 'agent') and self.agent and hasattr(self.agent, 'chat_context') and self.agent.chat_context:
+            try:
+                self.agent.chat_context.cleanup()
+                logger.info("Agent chat context cleaned up")
+            except Exception as e:
+                logger.error(f"Error cleaning up agent chat context: {e}")
+        
+        self.transcription_callback = None
+        self.user_speech_callback = None
+        self.stt = None
+        self.llm = None
+        self.tts = None
+        self.vad = None
+        self.turn_detector = None
+        self.agent = None
+        self.denoise = None
+        self._stt_started = False
+        self._partial_response = ""
+        self._is_interrupted = False
+        logger.info("Conversation flow cleaned up")
\ No newline at end of file
