@@ -4,6 +4,8 @@ import asyncio
 
 from .event_emitter import EventEmitter
 from .room.audio_stream import CustomAudioStreamTrack
+import logging
+logger = logging.getLogger(__name__)
 
 class Pipeline(EventEmitter[Literal["start"]], ABC):
     """
@@ -69,4 +71,22 @@ class Pipeline(EventEmitter[Literal["start"]], ABC):
         """
         Send a message to the pipeline.
         """
-        pass
\ No newline at end of file
+        pass
+    
+    async def cleanup(self) -> None:
+        """
+        Cleanup pipeline resources.
+        Base implementation - subclasses should override and call super().cleanup()
+        """
+        self.loop = None
+        self.audio_track = None
+        self._wake_up_callback = None
+        logger.info("Pipeline cleaned up")
+    
+    async def leave(self) -> None:
+        """
+        Leave the pipeline.
+        Base implementation - subclasses should override if needed.
+        """
+        logger.info("Leaving pipeline")
+        await self.cleanup()
\ No newline at end of file
